@@ -920,11 +920,18 @@ def _pkg_support_from_cache(x):
 def _cmp_gfx_alternatives(x, y):
     '''Compare two graphics driver names in terms of preference. (desktop)
 
-    NFB (New Feature Branch) or the latest is what we recommend on the desktop.
     -server always sorts after non-server.
     LTSB (Long Term Support Branch) always sorts before NFB (New Feature Branch).
     Legacy always sorts before Beta.
     '''
+    if x.endswith('-server') and not y.endswith('-server'):
+        return -1
+    if not x.endswith('-server') and y.endswith('-server'):
+        return 1
+    if _pkg_support_from_cache(x) == 'LTSB' and _pkg_support_from_cache(y) != 'LTSB':
+        return 1
+    if _pkg_support_from_cache(x) != 'LTSB' and _pkg_support_from_cache(y) == 'LTSB':
+        return -1
     if _pkg_support_from_cache(x) == 'Legacy' and _pkg_support_from_cache(y) != 'Legacy':
         return -1
     if _pkg_support_from_cache(x) != 'Legacy' and _pkg_support_from_cache(y) == 'Legacy':
@@ -933,10 +940,6 @@ def _cmp_gfx_alternatives(x, y):
         return -1
     if _pkg_support_from_cache(x) != 'Beta' and _pkg_support_from_cache(y) == 'Beta':
         return 1
-    if x.endswith('-server') and not y.endswith('-server'):
-        return -1
-    if not x.endswith('-server') and y.endswith('-server'):
-        return 1
     if x < y:
         return -1
     if x > y:
@@ -1125,8 +1128,10 @@ def get_linux_modules_metapackage(apt_cache, candidate):
         # dependencies
         reverse_deps = find_reverse_dependencies(apt_cache, linux_modules_match, 'linux-modules-nvidia-')
         pick = ''
+        modules_candidate = 'linux-modules-nvidia-%s-%s' % (candidate_flavour,
+                                                            get_linux_image(apt_cache).replace('linux-image-', ''))
         for dep in reverse_deps:
-            if dep > pick:
+            if dep == modules_candidate:
                 pick = dep
         if pick:
             metapackage = pick
